import importlib
import re
import yaml
import json
import os
import sys
from typing import Union, List, Dict
from yape.consts import YAPE_CONFIG_FILES
from yape.utils import deep_merge
from yape.log import yape_log


REQUIREMENTS_COLLECTION_NAME = "requirements"


class YAPEConfigRequirement(dict):
    @property
    def package(self) -> str:
        return self.get("package", None)

    @package.setter
    def package(self, val: str):
        self["package"] = val

    @property
    def import_path(self) -> str:
        """A path to a requirements file to import (relative to config root)"""
        return self.get("import_path", self.get("import", None))

    @classmethod
    def parse(cls, requirement: Union[str, dict, "YAPEConfigRequirement"]):
        if isinstance(requirement, cls):
            return requirement
        if isinstance(requirement, dict):
            return cls(requirement)

        assert isinstance(requirement, str), "Package dictionary must be a dict or string"
        return cls(package=requirement.strip())

    @classmethod
    def unique(cls, requirements: List[Union["YAPEConfigRequirement", dict]]):
        """Removes duplicate requirements and validates a requirement list."""
        requirements = [YAPEConfigRequirement.parse(r) for r in requirements]
        requirements.reverse()
        cleaned = []
        matched = set()
        for r in requirements:
            if r.import_path is not None:
                pkg_name = "import: " + r.import_path
            else:
                pkg_name = re.match(r"^[\w._-]+", r.package.strip())
                if pkg_name is None:
                    pkg_name = r.package
                else:
                    pkg_name = pkg_name[0]
            if pkg_name in matched:
                continue
            matched.add(pkg_name)
            cleaned.append(r)

        cleaned.reverse()
        return cleaned


class YAPEEnvironmentConfig(dict):
    @property
    def source_path(self) -> str:
        """The original path where the config was loaded (Overriden by load)"""
        return self.get("source_path", None)

    @source_path.setter
    def source_path(self, val: str):
        self["source_path"] = val

    @property
    def env_file(self) -> str:
        return self.get("env_file", ".env")

    @property
    def venv_directory(self) -> str:
        """The path of the virtual env directory"""
        return self.get("venv_directory", ".venv")

    @property
    def python_version(self) -> str:
        return self.get("python_version", None)

    @python_version.setter
    def python_version(self, val: str):
        self["python_version"] = val

    @property
    def python_executable(self) -> str:
        """The path to the python executable to use"""
        return self.get("python_executable", None)

    @property
    def requirements(self) -> List[YAPEConfigRequirement]:
        """A list of pip requirements"""
        requirements = self.get(REQUIREMENTS_COLLECTION_NAME, [])
        if any([not isinstance(r, YAPEConfigRequirement) for r in requirements]):
            requirements = [YAPEConfigRequirement.parse(r) for r in requirements]
            self[REQUIREMENTS_COLLECTION_NAME] = requirements
        return requirements

    @property
    def pip_install_args(self) -> List[str]:
        return self.get("pip_install_args", [])

    @property
    def venv_args(self) -> List[str]:
        return self.get("venv_args", [])

    def resolve_from_venv_directory(self, *parts: List[str]):
        return os.path.abspath(os.path.join(self.venv_path, *parts))

    def initialize_requirements(self):
        """Resolves the internal requirement imports and cleans up the requirements list"""
        if len(self.requirements) == 0:
            return []

        resolved_requirements = []
        src_path = None if self.source_path is None else os.path.dirname(self.source_path)
        for req in self.requirements:
            if req.import_path is not None:
                req_abs_path = (
                    os.path.join(src_path, req.import_path) if not os.path.isabs(req.import_path) else req.import_path
                )

                if os.path.isfile(req_abs_path):
                    with open(req_abs_path, "r", encoding="utf-8") as req_file:
                        requirements_raw = req_file.read()

                    # Clean comments
                    requirements_raw = re.sub(r"[#].*", "", requirements_raw)
                    for req_as_str in requirements_raw.split("\n"):
                        req_as_str = req_as_str.strip()
                        if len(req_as_str) == 0:
                            continue
                        resolved_requirements.append(YAPEConfigRequirement.parse(req_as_str))

            elif req.package is not None:
                resolved_requirements.append(req)

        self[REQUIREMENTS_COLLECTION_NAME] = resolved_requirements

    def initialize(
        self,
        resolve_imports: bool = True,
    ):
        """Initializes the config"""
        if resolve_imports:
            self.initialize_requirements()


class YAPEConfig(YAPEEnvironmentConfig):
    @property
    def source_directory(self) -> str:
        """The directory path to the source for the virtual environment (where the config is)"""
        return self.get("source_directory", None)

    @source_directory.setter
    def source_directory(self, val: str):
        self["source_directory"] = val

    @property
    def venv_path(self) -> str:
        if os.path.isabs(self.venv_directory):
            return self.venv_directory
        return os.path.abspath(os.path.join(self.source_directory, self.venv_directory))

    def resolve_from_source_directory(self, *parts: List[str]):
        return os.path.abspath(os.path.join(self.source_directory, *parts))

    @property
    def environments(self) -> Dict[str, YAPEEnvironmentConfig]:
        return self.get("environments", {})

    @property
    def inherit(self) -> bool:
        return self.get("inherit", False)

    def stop_inheritance(self) -> bool:
        """If returns true, stops any further inheritance.
        Overrideable function.
        """
        if self.inherit is not True:
            return True
        return False

    def load_virtualenv(self):
        """Loads the virtual environment into memory (using activate.py)."""
        import_path = self.resolve_from_venv_directory("bin", "activate_this.py")
        assert os.path.isfile(import_path), "Virtual env not found or virtualenv invalid @ " + self.venv_path
        spec = importlib.util.spec_from_file_location(re.sub(r"[^\w]+", "_", import_path), import_path)
        foo = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(foo)

    def initialize(
        self,
        source_directory: str = None,
        resolve_imports: bool = True,
        clean_duplicate_requirements: bool = True,
    ):
        super().initialize(
            resolve_imports=resolve_imports,
        )

        if clean_duplicate_requirements:
            self.clean_duplicate_requirements()

        if source_directory:
            self.source_directory = os.path.abspath(self.source_directory or source_directory)
        self.python_version = self.python_version or f"{sys.version_info.major}.{sys.version_info.minor}"

    def clean_duplicate_requirements(self):
        configs = [self] + list(self.environments.values())
        for c in configs:
            if REQUIREMENTS_COLLECTION_NAME in c:
                c[REQUIREMENTS_COLLECTION_NAME] = YAPEConfigRequirement.unique(c[REQUIREMENTS_COLLECTION_NAME])

    def to_dictionary(self) -> dict:
        return json.loads(json.dumps(self))

    def has_virtual_environment(self) -> dict:
        return os.path.isdir(self.venv_path)

    @classmethod
    def load(
        cls,
        config_path: str,
        environment: str = None,
        inherit_depth: int = -1,
        delete_environments: bool = True,
        resolve_imports: bool = True,
        clean_duplicate_requirements: bool = True,
    ):
        """Loads the YAPE environment configuration and initializes it.

        Args:
            config_path (str): The path/file to load the config from.
            environment (str, optional): The configuration environment to use. Defaults to None.
            inherit_depth (int, optional): If true, inherits configuration from parent directories. Defaults to True.
        """
        if config_path.endswith("/"):
            config_path = config_path[:-1]

        config_path = os.path.abspath(config_path)
        yape_log.debug("Reading configuration, starting from " + config_path)

        merge_configs: List[YAPEConfig] = []
        inherit_stopped = False

        def load_config_from_file(filepath):
            """Loads a config into the list.
            Returns false if needs to stop loading.
            """
            nonlocal inherit_stopped
            config = None
            filepath = os.path.abspath(filepath)
            if not os.path.isfile(filepath):
                return

            with open(filepath, "r", encoding="utf-8") as config_file:
                if filepath.endswith(".yaml") or filepath.endswith(".yml"):
                    config = yaml.safe_load(config_file.read())
                else:
                    config = json.loads(config_file.read())

            if config is not None and isinstance(config, dict):
                config = YAPEConfig(config)
                config.source_path = filepath
                merge_configs.append(YAPEConfig(config))
                if config.stop_inheritance():
                    inherit_stopped = True

            yape_log.debug("Loaded config file: " + filepath)

            return config

        config_filepath_groups: List[List[str]] = []

        if os.path.isfile(config_path):
            # If a direct file, this is not included in inheritance.
            load_config_from_file(config_path)
            config_path = os.path.dirname(config_path)
            yape_log.debug("Config path was a file, loading parent directory config " + config_path)

        source_directory = config_path

        while not inherit_stopped:
            config_filepath_groups.append(
                [os.path.join(config_path, filename) for filename in YAPE_CONFIG_FILES],
            )
            parent_path = os.path.dirname(config_path)
            if parent_path is None or parent_path == config_path:
                break
            config_path = parent_path

        if inherit_depth is not None and inherit_depth > -1:
            config_filepath_groups = config_filepath_groups[0 : inherit_depth + 1]  # noqa E203

        for fgroup in config_filepath_groups:
            for fpath in fgroup:
                load_config_from_file(fpath)

                if inherit_stopped:
                    break
            if inherit_stopped:
                break

        # reversing the config merge order
        merge_configs.reverse()

        environment_found = False

        for config in merge_configs:
            if environment is not None and environment in config.environments:
                environment_found = True
                deep_merge(config, config.environments[environment])

            config.initialize(resolve_imports=resolve_imports)

        assert environment is None or environment_found, ValueError(
            f"No environment named '{environment}' was found in the config (or parent configs)"
        )

        merged = YAPEConfig(deep_merge({}, *merge_configs))

        if delete_environments and "environments" in merged:
            del merged["environments"]

        merged.initialize(
            source_directory=source_directory,
            resolve_imports=resolve_imports,
            clean_duplicate_requirements=clean_duplicate_requirements,
        )

        yape_log.debug("Configuration loaded")

        return merged



# flake8: noqa
import os

ENTRY_ENVS = os.environ.copy()

from yape import consts  # noqa

consts.ENTRY_ENVS = ENTRY_ENVS

from yape.cli import run_cli_main  # noqa

if __name__ == "__main__":
    run_cli_main()



from yape.log import yape_log
from yape.utils import run_python_module, option_or_empty, clean_args
from yape.config import YAPEConfig


def virtualenv_args(config: YAPEConfig):
    """Returns the virtualenv args from the yape config

    Args:
        config (YAPEConfig): The yape config.
    """
    return clean_args(
        *option_or_empty("--python", config.python_executable or config.python_version),
        *config.venv_args,
        config.venv_path,
    )


def virtualenv_create(config: YAPEConfig):
    """Create a virtualenv given the yape config.

    Args:
        config (YAPEConfig): The yape config.
    """
    yape_log.info("Creating virtualenv @ " + config.venv_path)
    cmnd = ["virtualenv", *virtualenv_args(config)]
    yape_log.debug(str(cmnd))
    run_python_module(*cmnd, use_vevn=False)



import os
import logging

NO_COLOR = os.environ.get("NO_COLOR") == "true"


class YAPELogFormatter(logging.Formatter):
    colors = {
        "red": "\033[0;31m",
        "green": "\033[0;32m",
        "yellow": "\033[0;33m",
        "blue": "\033[0;34m",
        "magenta": "\033[0;35m",
        "cyan": "\033[0;36m",
        "gray": "\033[0;90m",
        "end": "\033[0m",
    }

    gray_format = "[%(asctime)s][%(levelname)8s] %(message)s"
    color_format = "{gray}[%(asctime)s]{end}{lvl_color}[%(levelname)8s]{end} %(message)s"

    FORMATS = {
        logging.DEBUG: "gray",
        logging.INFO: "green",
        logging.WARNING: "yellow",
        logging.ERROR: "red",
        logging.CRITICAL: "red",
    }

    def format(self, record):
        active_format = self.color_format if not NO_COLOR else self.gray_format
        lvl_color = self.colors[self.FORMATS.get(record.levelno, "gray")]
        active_format = active_format.format(lvl_color=lvl_color, **self.colors)
        formatter = logging.Formatter(active_format)
        return formatter.format(record)


yape_log = logging.getLogger("yape_log")
yape_log.setLevel(logging.getLevelName(os.environ.get("LOG_LEVEL", "INFO")))


# create console handler
yape_log_formatter = YAPELogFormatter()
yape_log_handler = logging.StreamHandler()
yape_log.addHandler(yape_log_handler)
yape_log_handler.setFormatter(yape_log_formatter)


if __name__ == "__main__":
    yape_log.info("Tester")



import enum
import json
import yaml
import shlex
from typing import Union


class PrintFormat(enum.Enum):
    list = "list"
    cli = "cli"
    yaml = "yaml"
    json = "json"


def get_print_formatted(
    format: PrintFormat,
    val: Union[list, dict],
    quote_cli: bool = True,
):
    if isinstance(val, dict) and (format == PrintFormat.list or format == PrintFormat.cli):
        as_list = []
        for k, v in val.items():
            as_list.append(k)
            as_list.append(v)
        val = as_list

    def print_list_value(v):
        if isinstance(v, list) or isinstance(v, dict):
            v = json.dumps(v)
        else:
            v = str(v)
        return v

    if format == PrintFormat.list:
        return "\n".join(print_list_value(v) for v in val)
    elif format == PrintFormat.cli:
        val = [print_list_value(v) for v in val]
        if quote_cli:
            val = [shlex.quote(v) for v in val]
        return " ".join(val)
    elif format == PrintFormat.yaml:
        return yaml.safe_dump(val)
    else:
        return json.dumps(val)



import os
from typing import List, Union
import click
from dotenv import load_dotenv
import yape.commands as yape_commands
from yape.log import yape_log
from yape.consts import YAPE_VERSION
from yape.format import PrintFormat, get_print_formatted
from yape.config import YAPEConfig
from yape.utils import resolve_path


class CommonOptions(dict):
    SHOW_FULL_ERRORS = None

    @property
    def path(self) -> str:
        return self.get("path", None)

    @property
    def environment(self) -> str:
        return self.get("env", None)

    @property
    def inherit_depth(self) -> int:
        return self.get("inherit_depth", None)

    @property
    def env_file(self) -> str:
        return self.get("env_file", os.environ.get("YAPE_ENV_FILE", ".env"))

    def load(self, resolve_imports: bool = True) -> YAPEConfig:
        env_file = resolve_path(self.env_file)
        if os.path.isfile(env_file):
            yape_log.debug("Loading environment variables from: " + env_file)
            load_dotenv(env_file)

        config = YAPEConfig.load(
            self.path,
            environment=self.environment,
            inherit_depth=self.inherit_depth,
            resolve_imports=resolve_imports,
        )

        if os.path.isfile(env_file):
            yape_log.debug("Loading environment variables from: " + env_file)
            load_dotenv(env_file)

        return config

    @classmethod
    def decorator(cls):
        def apply(fn):
            for opt in [
                click.argument("path", default=os.curdir),
                click.option(
                    "-e",
                    "--env",
                    "--environment",
                    help="Name of the extra environment config to load",
                    default=None,
                ),
                click.option("--env-file", help="The yape environment local env file", default=".env"),
                click.option(
                    "--inherit-depth",
                    help="Max number of config parents to inherit (0 to disable, -1 inf)",
                    default=None,
                    type=int,
                ),
                click.option("--full-errors", help="Show full python errors", is_flag=True),
            ]:
                fn = opt(fn)
            return fn

        return apply


class FormatOptions(dict):
    @property
    def quote(self) -> bool:
        return self.get("quote", False)

    @property
    def format(self) -> PrintFormat:
        return self.get("format", PrintFormat.cli)

    def print(self, val: Union[list, dict]):
        return get_print_formatted(self.format, val, self.quote)

    @classmethod
    def decorator(cls, default_format: PrintFormat = PrintFormat.cli):
        def apply(*args):
            for opt in [
                click.option(
                    "--format",
                    help=f"The document formate to print in ({', '.join(k.value for k in PrintFormat)})",
                    type=PrintFormat,
                    default=default_format,
                ),
                click.option("--quote", help="Quote cli arguments if needed", is_flag=True, default=False),
            ]:
                fn = opt(*args)
            return fn

        return apply


@click.group(help=f"Yet Another Python Environment manager\nversion: {YAPE_VERSION}")
def yape():
    pass


@yape.command("version", help="Show the yape version")
def version():
    print(YAPE_VERSION)


@yape.group("pip", help="Run pip commands through yape")
def pip_command():
    pass


@pip_command.command("args", help="Create a pip install command for the yape config")
@FormatOptions.decorator(PrintFormat.cli)
@CommonOptions.decorator()
def pip_args(**kwargs):
    config = CommonOptions(kwargs).load()
    print(FormatOptions(kwargs).print(yape_commands.pip_command_args(config)))


@pip_command.command("install", help="Run pip install using the config (within this python version)")
@CommonOptions.decorator()
def pip_install(**kwargs):
    config = CommonOptions(kwargs).load()
    yape_commands.pip_install(config)


@yape.group("virtualenv", help="Run virtualenv commands through yape")
def virtualenv_command():
    pass


@virtualenv_command.command("args", help="Create a venv install command for the yape config")
@FormatOptions.decorator(PrintFormat.cli)
@CommonOptions.decorator()
def virtualenv_args(**kwargs):
    config = CommonOptions(kwargs).load()
    print(FormatOptions(kwargs).print(yape_commands.virtualenv_args(config)))


@virtualenv_command.command("create", help="Create a venv install command for the yape config")
@FormatOptions.decorator(PrintFormat.cli)
@CommonOptions.decorator()
def virtualenv_create(**kwargs):
    config = CommonOptions(kwargs).load()
    yape_commands.virtualenv_create(config)


@yape.command("init", help="Initializes the yape configuration in a folder")
@click.option("-p", "--python-version", help="Use this python version", default=None)
@click.option("-f", "--config-filename", help="Override the configuration filename", default=None)
@click.option("--no-requirement-files", help="Do not initialize with requirement files", is_flag=True, default=None)
@click.option("--reset", help="Delete current configuration and reset it.", is_flag=True, default=False)
@CommonOptions.decorator()
def init(
    reset=False,
    python_version: str = None,
    config_filename: str = None,
    no_requirement_files: bool = False,
    **kwargs,
):
    config = CommonOptions(kwargs).load(resolve_imports=False)
    yape_commands.init(
        active_config=config,
        merge_with_current=not reset,
        python_version=python_version,
        config_filename=config_filename,
        add_requirement_files=not no_requirement_files,
    )
    yape_log.info("yape configuration initialized @ " + config.source_directory)


@yape.command("delete", help="Delete the virtual environment installation")
@click.option("-f", "--force", help="Do not confirm the operation", is_flag=True, default=False)
@CommonOptions.decorator()
def delete(force: bool = False, **kwargs):
    config = CommonOptions(kwargs).load()
    yape_commands.delete(config, force=force)


@yape.command(help="Export the requirement list")
@FormatOptions.decorator(PrintFormat.list)
@CommonOptions.decorator()
def export(**kwargs):
    config = CommonOptions(kwargs).load()
    packages = [r.package for r in config.requirements if r.package is not None]
    print(FormatOptions(kwargs).print(packages))


@yape.command(help="Print the YAPE computed configuration")
@click.option("--resolve", help="Resolve requirement files", is_flag=True, default=None)
@FormatOptions.decorator(PrintFormat.yaml)
@CommonOptions.decorator()
def config(resolve: bool = False, **kwargs):
    config = CommonOptions(kwargs).load(resolve_imports=resolve)
    print(FormatOptions(kwargs).print(config.to_dictionary()))


@yape.command(help="Start a venv shell")
@CommonOptions.decorator()
@click.option("-k", "--keep-current-directory", help="Don't move into the venv directory", is_flag=True, default=False)
def shell(keep_current_directory: bool = False, **kwargs):
    config = CommonOptions(kwargs).load()
    yape_commands.shell(config, use_source_dir=not keep_current_directory)


@yape.command(
    help="Run a command inside the venv shell",
    context_settings={"ignore_unknown_options": True, "allow_extra_args": True},
)
@click.option("-k", "--keep-current-directory", help="Don't move into the venv directory", is_flag=True, default=False)
@click.option("-p", "--path", help="The path to the source dir of the environment", default=os.curdir)
@click.argument("command")
@click.argument("args", nargs=-1)
def run(command: str, args: List[str] = [], keep_current_directory: bool = False, **kwargs):
    config = CommonOptions(kwargs).load()
    config.load_virtualenv()
    cmnd = [command] + list(args)
    yape_commands.handover(config, *cmnd, use_source_dir=not keep_current_directory)


@yape.command()
@CommonOptions.decorator()
def install(**kwargs):
    config = CommonOptions(kwargs).load()
    yape_commands.install(config)


def run_cli_main():
    import sys

    if "--full-errors" in sys.argv:
        CommonOptions.SHOW_FULL_ERRORS = True

    try:
        yape()
    except Exception as ex:
        if CommonOptions.SHOW_FULL_ERRORS is None:
            CommonOptions.SHOW_FULL_ERRORS = os.environ.get("YAPE_FULL_ERRORS", "false").lower() == "true"
        if CommonOptions.SHOW_FULL_ERRORS:
            raise ex
        else:
            yape_log.error(ex)
            exit(1)


if __name__ == "__main__":
    run_cli_main()



import os
from yape.commands.virtualenv import virtualenv_create
from yape.config import YAPEConfig
from yape.consts import ENTRY_ENVS
from yape.log import yape_log


def handover(
    config: YAPEConfig,
    *command: str,
    use_source_dir: bool = True,
    env: dict = None,
):
    """Replaces the current executing process with the executing command.

    Args:
        config (YAPEConfig): The yape config
        use_source_dir (bool, optional): If true, then use the config venv dir to start the process. Defaults to True.
        shell_executable (str, optional): The shell executable to use. Defaults to None.
    """
    assert config.has_virtual_environment(), "Could not find virtual environment @ " + config.venv_path

    if use_source_dir:
        os.chdir(config.source_directory)

    command = list(command)

    yape_log.debug(f"Running: {command}")
    os.execvpe(command[0], command, env=env or os.environ.copy())


def shell(
    config: YAPEConfig,
    use_source_dir: bool = False,
    active_shell: str = None,
):
    """Start a yape shell, with the venv enabled.

    Args:
        config (YAPEConfig): The yape config
        use_source_dir (bool, optional): If true, then use the config venv dir to start the process. Defaults to True.
        shell_executable (str, optional): The shell executable to use. Defaults to None.
    """

    if not config.has_virtual_environment():
        yape_log.warning("Virtual env not found")
        if input("Create? (y/n)") != "y":
            yape_log.info("Aborted")
            return

        virtualenv_create(config)

    # Replacing current process with new shell.
    command = []
    if os.name != "nt":
        active_shell = active_shell or os.environ.get("SHELL", "sh")
        yape_activate = config.resolve_from_venv_directory("bin", "activate_yape_shell")
        venv_activate = config.resolve_from_venv_directory("bin", "activate")
        command = [active_shell, yape_activate, venv_activate, active_shell]
    else:
        config.load_virtualenv()
        active_shell = active_shell or "cmd.exe"
        command = [active_shell]

    handover(
        config,
        active_shell,
        *command,
        env=ENTRY_ENVS,
        use_source_dir=use_source_dir,
    )



from yape.log import yape_log
from yape.utils import run_python_module, clean_args
from yape.config import YAPEConfig


def pip_command_args(config: YAPEConfig):
    """Return the yape pip install args (for cli)

    Args:
        config (YAPEConfig): The yape config.
    """
    return clean_args(
        "install",
        *config.pip_install_args,
        *[r.package for r in config.requirements],
    )


def pip_install(config: YAPEConfig):
    """Run the pip install module in the yape virtual env.

    Args:
        config (YAPEConfig): The yape config.
    """
    assert len(config.requirements) > 0, "No requirements found in config, cannot install."
    yape_log.info("Running pip install in venv @ " + config.venv_path)
    config.load_virtualenv()
    cmnd = ["pip", *pip_command_args(config)]
    yape_log.debug(str(cmnd))
    run_python_module(*cmnd, use_vevn=True)



import json
import os
import shutil
from typing import List
import yaml
from yape.log import yape_log
from yape.consts import YAPE_CONFIG_FILES
from yape.config import YAPEConfig
from yape.utils import deep_merge, resolve_template, touch
from yape.commands.virtualenv import virtualenv_create
from yape.commands.pip import pip_install


def delete(config: YAPEConfig, force: bool = False):
    if config.has_virtual_environment():
        if not force:
            yape_log.warn("You are about to delete the virtual environment @ " + config.venv_path)
            if input("WARNING: are you sure? (y/n)") != "y":
                yape_log.info("Aborted")
                return
        shutil.rmtree(config.venv_path)
        yape_log.info("Delete virtual environment folder @ " + config.venv_path)
    else:
        yape_log.warn("No virtual environment @ " + config.venv_path)


def init(
    active_config: YAPEConfig,
    config_filename: str = None,
    python_version: str = None,
    merge_with_current: bool = False,
    add_requirement_files: bool = True,
):
    # Checking configuration
    to_merge: List[YAPEConfig] = []
    to_merge.append(
        YAPEConfig.load(
            resolve_template("config.yaml"),
            inherit_depth=0,
            resolve_imports=False,
            delete_environments=False,
            clean_duplicate_requirements=False,
        )
    )
    if add_requirement_files:
        to_merge.append(
            YAPEConfig.load(
                resolve_template("config_with_requirements.yaml"),
                inherit_depth=0,
                resolve_imports=False,
                delete_environments=False,
                clean_duplicate_requirements=False,
            )
        )

    if merge_with_current:
        to_merge.append(active_config)

    config = YAPEConfig(deep_merge({}, *to_merge))
    config.initialize(resolve_imports=False)

    if python_version is not None:
        config.python_version = python_version

    # Deleting invalid keys
    def del_key(key: str):
        if key in config:
            del config[key]

    del_key("source_path")
    del_key("source_directory")
    if config.python_executable in config:
        del_key("python_version")

    config_filename = config_filename or YAPE_CONFIG_FILES[0] or ".yape.yaml"
    with open(os.path.join(active_config.source_directory, config_filename), "w") as config_file:
        if config_filename.endswith(".json"):
            config_file.write(json.dumps(config.to_dictionary(), indent=2))
        else:
            config_file.write(yaml.safe_dump(config.to_dictionary()))

    if add_requirement_files:
        touch(os.path.join(active_config.source_directory, "requirements.txt"))
        touch(os.path.join(active_config.source_directory, "requirements.dev.txt"))


def install(config: YAPEConfig):
    if not config.has_virtual_environment():
        virtualenv_create(config)

    if len(config.requirements) > 0:
        pip_install(config)
    else:
        yape_log.warn("No requirements found in config. Skipping pip install")

    yape_log.info("Copying yape shell activation script")
    shutil.copyfile(
        resolve_template("activate_yape_shell"),
        config.resolve_from_venv_directory("activate_yape_shell"),
    )

    yape_log.info("Venv ready")



import os
import subprocess
import sys
from typing import List, Union


def option_or_empty(key, val):
    if val is None:
        return []
    return [key, val]


def clean_args(*args: List[str]):
    return [str(a) for a in args if a is not None and len(a) > 0]


def touch(fname):
    if os.path.exists(fname):
        os.utime(fname, None)
    else:
        open(fname, "a").close()


def resolve_template(*path: str):
    return resolve_path(
        *path,
        root_directory=os.path.join(os.path.dirname(__file__), "templates"),
    )


def resolve_path(*path_parts: str, root_directory: str = None):
    path_parts = [p.strip() for p in path_parts if p is not None and len(p.strip()) > 0]
    assert len(path_parts) > 0, ValueError("You must provide at least one path part")
    root_directory = root_directory or os.curdir
    if not path_parts[0].startswith("/"):
        path_parts = [root_directory] + path_parts
    return os.path.abspath(os.path.join(*path_parts))


def deep_merge(target: Union[dict, list], *sources):
    if isinstance(target, list):
        assert all(isinstance(src, list) for src in sources), (
            "Merge target and source must be of the same type (list)",
        )
        # List merges as concat, and dose not merge the internal values.
        for src in sources:
            target += src
        return target

    if isinstance(target, dict):
        assert all(isinstance(src, dict) for src in sources), (
            "Merge target and source must be of the same type (dict)",
        )

        for src in sources:
            for key in src.keys():
                if key not in target:
                    target[key] = src[key]
                    continue
                if isinstance(src[key], list) and isinstance(target[key], list):
                    target[key] = deep_merge([], target[key], src[key])
                elif isinstance(src[key], dict) and isinstance(target[key], dict):
                    target[key] = deep_merge({}, target[key], src[key])
                else:
                    target[key] = src[key]
    return target


def run_python_module(
    module_name,
    *args,
    envs: dict = {},
    throw_errors: bool = True,
    include_process_envs: bool = True,
    executable: str = None,
    use_vevn: bool = True,
):
    if executable is None:
        if use_vevn and os.environ.get("VIRTUAL_ENV", None) is not None:
            executable = "python"
        else:
            executable = sys.executable

    return run_shell_command(
        executable,
        "-m",
        module_name,
        *args,
        envs=envs,
        throw_errors=throw_errors,
        include_process_envs=include_process_envs,
    )


def run_shell_command(
    *cmnd: List[str],
    envs: dict = {},
    throw_errors: bool = True,
    include_process_envs: bool = True,
):
    return run_shell_commands(
        [cmnd],
        envs=envs,
        throw_errors=throw_errors,
        include_process_envs=include_process_envs,
    )


def run_shell_commands(
    commands: List[List[str]],
    seperator: str = "&&",
    envs: dict = {},
    throw_errors: bool = True,
    include_process_envs: bool = True,
):
    # Compose the command
    shell_command = []
    is_first = True
    for cmnd in commands:
        if not is_first:
            shell_command.append(seperator)
        is_first = False
        for part in cmnd:
            shell_command.append(part)

    run_env = os.environ.copy() if include_process_envs else {}
    run_env.update(envs or {})

    rslt = subprocess.run(
        " ".join(shell_command),
        shell=True,
        env=run_env,
    )

    if throw_errors and rslt.returncode != 0:
        raise subprocess.SubprocessError(rslt.stderr)

    return rslt



import os
import re
from setuptools import setup, find_packages
import logging

REPO_PATH = os.path.dirname(os.path.abspath(__file__))
VERSION_PATH = os.path.abspath(os.environ.get("VERSION_PATH", os.path.join(REPO_PATH, ".version")))
GITHUB_URL = "https://github.com/LamaAni/yape.git"

packages = find_packages()


# Get the long description from the README file
with open(os.path.join(REPO_PATH, "README.md"), "r") as readme:
    long_description = readme.read()

with open(os.path.join(REPO_PATH, "requirements.txt"), "r") as requirements_file:
    requirements_text = requirements_file.read()
    requirements_text = re.sub(r"[#].*", "", requirements_text)
    requirement_list = [r.strip() for r in requirements_text.split("\n") if len(r.strip()) > 0]

version = None
if os.path.isfile(VERSION_PATH):
    with open(VERSION_PATH, "r") as file:
        version = file.read()
else:
    logging.info("Version file not found @ " + VERSION_PATH)
    logging.info("Using default version debug")

version = version or os.environ.get("SETUP_VERSION", "debug")

print(packages)

setup(
    name="yapenv",  # Required
    version=version,  # Required
    description="Yet another python environment",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=GITHUB_URL,
    author="Zav Shotan",
    keywords="environment, venv, virtualenv, pyenv, poetry",
    packages=packages,
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "yape=yape.__main__:run_cli_main",
            "yapenv=yape.__main__:run_cli_main",
        ],
    },
    install_requires=requirement_list,
    project_urls={
        "Source": GITHUB_URL,
    },
)



import os
import re

ENTRY_ENVS = os.environ.copy()
YAPE_CONFIG_FILES = re.split(r"[\s,]+", os.environ.get("YAPE_CONFIG_FILES", ".yape.yaml .yape .yape.yml .yape.json"))


def get_version():
    version_path = os.path.join(os.path.dirname(__file__), ".version")
    if os.path.isfile(version_path):
        with open(version_path, "r") as raw:
            return raw.read().strip()
    return "local"


YAPE_VERSION = get_version()
__version__ = YAPE_VERSION
